/**
 * 
 */
package org.bonitasoft.connector;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;

import org.bonitasoft.engine.connector.ConnectorException;
import org.bonitasoft.engine.connector.ConnectorValidationException;

import com.google.api.client.auth.oauth2.Credential;
import com.google.api.client.googleapis.auth.oauth2.GoogleCredential;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.jackson2.JacksonFactory;
import com.google.api.client.util.DateTime;
import com.google.api.services.calendar.Calendar;
import com.google.api.services.calendar.model.Events;

/**
 * The connector execution will follow the steps 1 - setInputParameters() -->
 * the connector receives input parameters values 2 - validateInputParameters()
 * --> the connector can validate input parameters values 3 - connect() --> the
 * connector can establish a connection to a remote server (if necessary) 4 -
 * executeBusinessLogic() --> execute the connector 5 - getOutputParameters()
 * --> output are retrieved from connector 6 - disconnect() --> the connector
 * can close connection to remote server (if any)
 */
public class RetrieveEventByRangeImpl extends AbstractRetrieveEventByRangeImpl {

	
	Logger log = Logger.getLogger("org.bonitasoft.connector.google");
	
	public void validateInputParameters()
		    throws ConnectorValidationException
		  {
		    List errors = new ArrayList();
		    if (getServiceAccountId() == null)
		      errors.add("Service Account ID must be set.");
		    else if (getServiceAccountId().isEmpty()) {
		      errors.add("Service Account ID must not be an empty String.");
		    }
		    if (getP12ServiceAccountFile() == null) {
		      errors.add("Service Account P12 File must be set.");
		    } else if (getP12ServiceAccountFile().isEmpty()) {
		      errors.add("Service Account P12 File must not be en empty String.");
		    } else {
		      File p12File = new File(getP12ServiceAccountFile());
		      if (!p12File.exists())
		        errors.add("Service Account P12 File refers to a non existing file: " + getP12ServiceAccountFile() + ".");
		      else if (p12File.isDirectory()) {
		        errors.add("Service Account P12 File refers to a directory and not a file: " + getP12ServiceAccountFile() + ".");
		      }

		    }

		    if (getAgendaId() == null) {
		      errors.add("CalendarId must be set.");
		    }
		    if (!errors.isEmpty())
		      throw new ConnectorValidationException(this, errors);
		  }
	
	@Override
	protected void executeBusinessLogic() throws ConnectorException {

		try {
			HttpTransport httpTransport = new NetHttpTransport();
			try {
				JsonFactory jsonFactory = new JacksonFactory();

				GoogleCredential.Builder credentialBuilder = new GoogleCredential.Builder();
				credentialBuilder.setTransport(httpTransport);
				credentialBuilder.setJsonFactory(jsonFactory);
				credentialBuilder.setServiceAccountId(getServiceAccountId());
				credentialBuilder
						.setServiceAccountPrivateKeyFromP12File(new File(
								getP12ServiceAccountFile()));
				if (getServiceAccountUser() != null) {
					credentialBuilder
							.setServiceAccountUser(getServiceAccountUser());
				}
				credentialBuilder.setServiceAccountScopes(Collections
						.singleton("https://www.googleapis.com/auth/calendar"));
				Credential credential = credentialBuilder.build();

				Calendar calendar = new Calendar.Builder(httpTransport,
						jsonFactory, credential).setApplicationName(
						getApplicationName()).build();

				doJobWithCalendarEvents(calendar.events());
			} finally {
				httpTransport.shutdown();
			}
		} catch (Exception e) {
			throw new ConnectorException(e);
		}

	}

	@Override
	public void connect() throws ConnectorException {
		// [Optional] Open a connection to remote server

	}

	@Override
	public void disconnect() throws ConnectorException {
		// [Optional] Close connection to remote server

	}

	private void doJobWithCalendarEvents(Calendar.Events events)
			throws Exception {
		DateTime minTime = new DateTime(getBeginDate());
		DateTime maxTime = new DateTime(getEndDate());
		Calendar.Events.List get = events.list(getAgendaId()).setTimeMin(minTime).setTimeMax(maxTime);
		
		setOutputParameters((Events) get.execute());
	}

	protected void setOutputParameters(Events events) {
		//log.severe("events.getItems() >>"+events.getItems().size());
		if (events != null) {
			List eventList = events.getItems();
			setOutputParameter("listEvents", eventList);
		}
	}
	
	
}
